/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Gareth Johnson
 */

define(["bajaScript/baja/sys/inherit",
        "bajaScript/baja/comm/ServerError"], function (
         inherit,
         ServerError) {
  "use strict";
  
  var subclass = inherit.subclass,
      defaultErrorMessage = "Default Message";
  
  /**
   * A BOX Error.
   * 
   * A BOX Error is one that originated from the Server using the BOX protocol.
   *
   * @class
   * @alias baja.comm.BoxError
   * @private
   * @extends baja.comm.ServerError
   * @param [errorType] the type of error
   * @param [obj]
   * @param [obj.c] the error class name
   * @param [obj.m] the error message
   * @param [obj.s] the error stack trace
   */  
  var BoxError = function (errorType, obj) {
    obj = obj || {};

    ServerError.call(this, obj.m || defaultErrorMessage);
    this.name = errorType;

    
    /**
     * The name of the Java class that originated the error station-side
     * @type {String}
     */
    this.javaClass = obj.c;

    /**
     * The full stack trace of the Java exception station-side
     * @type {String}
     */
    this.javaStackTrace = obj.s;
  };
   
  subclass(BoxError, ServerError);
  
  return BoxError;
});