/**
 * @copyright 2018 Tridium, Inc. All Rights Reserved.
 * @author Gareth Johnson
 */

/**
 * Defines {@link baja.IpScheme}.
 * @module baja/ord/IpScheme
 */
define(["bajaScript/sys",
        "bajaScript/baja/ord/OrdQuery",
        "bajaScript/baja/ord/OrdScheme",
        "bajaScript/baja/ord/OrdTarget",
        "bajaScript/baja/ord/ordUtil"], 
    function (baja, OrdQuery, OrdScheme, OrdTarget, ordUtil) {
  
  "use strict";
  
  var subclass = baja.subclass,
      callSuper = baja.callSuper,
      trimToStart = ordUtil.trimToStart;
  
  /**
   * IP Host ORD Scheme.
   * 
   * This scheme has been added so ORDs can be normalized
   * and relativized correctly. Resolving this ORD will result in an
   * error as it's not currently supported in BajaScript.
   *
   * @class
   * @alias baja.IpScheme
   * @extends baja.OrdScheme
   * @private
   */    
  var IpScheme = function IpScheme() {
    callSuper(IpScheme, this, arguments);
  };
  
  subclass(IpScheme, OrdScheme);
  
  /**
   * Default IP ORD Scheme instance.
   * @private
   * @type {baja.IpScheme}
   */
  IpScheme.DEFAULT = new IpScheme();
  
  /**
   * Called when an ORD is resolved.
   *
   * @private
   *
   * @see baja.OrdScheme#resolve
   *
   * @param {module:baja/ord/OrdTarget} target  the current ORD Target.
   * @param {baja.OrdQuery} query  the ORD Query used in resolving the ORD.
   * @param {module:baja/ord/OrdQueryListCursor} cursor  the ORD Query List 
   * cursor used for helping to asynchronously resolve the ORD.
   * @param {Object} options  options used for resolving an ORD.
   */
  IpScheme.prototype.resolve = function (target, query, cursor, options) {
    throw new Error('IP ORD scheme unsupported');
  };
  
  /**
   * Return an ORD Query for the scheme.
   *
   * @returns {baja.OrdQuery}
   */
  IpScheme.prototype.parse = function (schemeName, body) {
    return new OrdQuery({
      scheme: this,
      schemeName: schemeName,
      body: body,
      isHost: true,
      isSession: false,
      normalize: trimToStart
    });
  };

  return IpScheme;
});