/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Danesh Kamal
 */

/**
 * @module nmodule/alarm/rc/db/AlarmPropertyColumn
 * @private
 */
define([
    'baja!',
    'jquery',
    'nmodule/alarm/rc/util/alarmUtils',
    'nmodule/webEditors/rc/fe/fe',
    'nmodule/webEditors/rc/wb/table/model/columns/PropertyColumn',
    'nmodule/alarm/rc/console/baja/BajaDataChannel'
  ], function (baja, $, alarmUtils, fe, PropertyColumn, BajaDataChannel) {

  "use strict";

  var AlarmPropertyColumn = function (property, map, params) {
    PropertyColumn.apply(this, [property, params]);
    this.$map = map;
  };

  AlarmPropertyColumn.prototype = Object.create(PropertyColumn.prototype);
  AlarmPropertyColumn.prototype.constructor = AlarmPropertyColumn;

  /**
   * Customize the column value for each row
   * @param {module:nmodule/webEditors/rc/wb/table/model/Row} row
   * @param {JQuery} dom
   * @returns {Promise} Promise resolved with display string
   */
  AlarmPropertyColumn.prototype.buildCell = function (row, dom) {
    var that         = this,
        name         = this.getName(),
        alarm        = row.getSubject(),
        str,
        timestampStr,
        timestampJq;

    return PropertyColumn.prototype.buildCell.apply(that, arguments).then(function () {
        str = dom.text();
        switch (name) {
          case 'alarmClass':
          case 'sourceState':
          case 'ackState':
          case 'alarmTransition':
            return alarmUtils.getAlarmDataDisplayName(str);

          case 'source':
            var sourceFormatPattern = alarm.getAlarmData().get('sourceName');
            if (sourceFormatPattern) {
              return baja.Format.format({ pattern: sourceFormatPattern }).then(function (str) {
                return dom.text(str);
              });
            }
            break;

          case 'timestamp':
          case 'normalTime':
          case 'ackTime':
          case 'lastUpdate':
            var tz = row.getSubject().alarmData && row.getSubject().alarmData.TimeZone || baja.TimeZone.UTC;
            timestampStr = BajaDataChannel.createDateString(that.getValueFor(row).getMillis(), tz);
            timestampJq = $("<span>" + timestampStr + "</span>");

            return dom.empty().append(timestampJq);
        }
      }
    );
  };

  /**
   * Override getValueFor() to permit BStruct subjects (specifically BAlarmRecord) *
   * @param row
   * @returns {*}
   */
  AlarmPropertyColumn.prototype.getValueFor = function (row) {
    var comp = row.getSubject(),
        prop = this.$prop,
        type = this.$type;

    if (comp.has(prop)) {
      return comp.get(prop);
    }

    if (type) {
      return baja.$(type);
    }

    throw new Error('component does not have property ' + prop + ' and no type given');
  };

  return (AlarmPropertyColumn);
});
