/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gareth Johnson
 */

/*jshint browser: true */

/**
 * WebSocket Connection.
 *
 * @module env/WebSocketConnection
 * @private
 */
define(["bajaScript/comm",
        "bajaScript/baja/comm/Callback",
        "bajaScript/env/Connection"], function (
        baja, 
        Callback,
        Connection) {

  "use strict";

  /**
   * A connection that uses a WebSocket as a connection.
   *
   * @class
   * @private
   * @extends Connection
   * @param {Object} handlers Holds a collection of functions used as asynchronous callbacks.
   */
  var WebSocketConnection = function (handlers) {
    baja.callSuper(WebSocketConnection, this, arguments);
  };

  baja.subclass(WebSocketConnection, Connection);

  /**
   * Make a new server session. This must be done using the default HTTP
   * mechanism before a web socket can be opened.
   * @return {Promise} promise to be resolved after the server session is
   * created
   */
  WebSocketConnection.prototype.doAuthenticate = function () {
    var cb = new Callback();
    baja.comm.makeServerSession(cb);
    return cb.promise();
  };

  /**
   * Opens a new web socket to the station.
   * @param {function} [WebSocket=window.WebSocket] the WebSocket constructor
   * to use
   */
  WebSocketConnection.prototype.start = function (WebSocket) {
    // Attempt to open a new BOX Web Socket.
    var that = this,
        handlers = that.$handlers,
        name,
        WebSocketCtor = (WebSocket || window.WebSocket),
        connection = that.$internalConnection = new WebSocketCtor((window.location.protocol && 
      /^https/i.test(window.location.protocol) ? "wss://" : "ws://") + 
      window.location.host + "/wsbox");

    for (name in handlers) {
      if (handlers.hasOwnProperty(name)) {
        connection.addEventListener(name, handlers[name], false);  
      }
    }
  };

  return WebSocketConnection;
});