/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * @private
 * @module nmodule/webEditors/rc/fe/baja/AbsTimeRangeEditor
 */
define(['lex!webEditors',
        'jquery',
        'Promise',
        'nmodule/webEditors/rc/fe/baja/ComplexCompositeEditor'], function (
         lexs,
         $,
         Promise,
         ComplexCompositeEditor) {

  'use strict';

  var webEditorsLex = lexs[0];

  function startBeforeEnd(startTime, endTime) {
    if (startTime.getJsDate().getTime() >= endTime.getJsDate().getTime()) {
      throw new Error(webEditorsLex.get('AbsTimeRangeEditor.startAfterEnd'));
    }
  }

  /**
   * Editor for an `AbsTimeRange`.
   *
   * An `AbsTimeRangeEditor` supports the following `bajaux` `Properties`:
   *
   * - `validateRange`: set to `true` to add a validate step that ensures the
   *   `startTime` slot is before, or equal to, the `endTime` slot.
   *
   * @private
   * @class
   * @extends module:nmodule/webEditors/rc/fe/baja/ComplexCompositeEditor
   * @alias module:nmodule/webEditors/rc/fe/baja/AbsTimeRangeEditor
   */
  var AbsTimeRangeEditor = function AbsTimeRangeEditor(params) {
    var that = this;

    ComplexCompositeEditor.call(that, $.extend({
      keyName: 'AbsTimeRangeEditor'
    }, params));

    if (that.properties().getValue('validateRange') === true) {
      that.validators().add(function (diff) {
        var value = that.value();
        return Promise.join(
          diff.getValue(value, 'startTime'),
          diff.getValue(value, 'endTime')
        )
          .spread(startBeforeEnd);
      });
    }
  };
  AbsTimeRangeEditor.prototype = Object.create(ComplexCompositeEditor.prototype);
  AbsTimeRangeEditor.prototype.constructor = AbsTimeRangeEditor;

  /**
   * Use slots `startTime` and `endTime`.
   *
   * @returns {Array}
   */
  AbsTimeRangeEditor.prototype.getSlotFilter = function () {
    return [ 'startTime', 'endTime' ];
  };

  return AbsTimeRangeEditor;
});
