/**
 * @copyright 2016 Tridium, Inc. All Rights Reserved.
 */

/**
 * @private
 * @module nmodule/webEditors/rc/util/textUtils
 */
define(['underscore'], function (_) {

  'use strict';

  /**
   * @private
   * @exports nmodule/webEditors/rc/util/textUtils
   */
  var exports = {};

  function isUpperCase(c) {
    return c === c.toUpperCase();
  }

  /**
   * Return the string parameter with the first letter capitalized, and subsequent
   * upper case letters preceded by a space character. This is intended to transform
   * a programmatic name to a more user friendly style. Will turn the string 'fooBar'
   * to the string 'Foo Bar', for instance. A non-truthy parameter value will result
   * in an empty string.
   *
   * @param {String} str
   * @returns {string}
   */
  exports.toFriendly = function (str) {
    if (!str) { return ''; }
    var initial = str.substring(0, 1).toUpperCase();
    return _.reduce(str.substring(1), function (memo, c) {
      return memo + (isUpperCase(c) ? ' ' + c : c);
    }, initial);
  };

  return exports;
});
