/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * @private
 * @module nmodule/webEditors/rc/wb/mgr/MgrSheet
 */
define(['Promise',
        'bajaux/Properties',
        'nmodule/webEditors/rc/wb/PropertySheet',
        'nmodule/webEditors/rc/wb/mgr/model/MgrModelCompositeBuilder'], function (
         Promise,
         Properties,
         PropertySheet,
         MgrModelCompositeBuilder) {

  'use strict';

  var MgrSheet = function MgrSheet(params) {
    var builder = params && params.data && params.data.builder;

    if (!(builder instanceof MgrModelCompositeBuilder)) {
      throw new Error('MgrModelCompositeBuilder required');
    }

    this.$baseBuilder = builder;

    PropertySheet.apply(this, arguments);
  };
  MgrSheet.prototype = Object.create(PropertySheet.prototype);
  MgrSheet.prototype.constructor = MgrSheet;

  MgrSheet.prototype.makeBuilder = function () {
    var that = this,
        builder = that.$baseBuilder,
        getConfigFor = builder.getConfigFor;

    builder.getDomFor = function (key) {
      return that.$buildSubEditorDom(key);
    };

    builder.getConfigFor = function (key) {
      return Promise.join(
        getConfigFor.apply(this, arguments), that.getOrdBase()
      )
        .spread(function (config, ordBase) {
          var type = config.type,
              RowCtor = that.$getRowConstructor();

          config.data = {
            builder: builder, key: key, displayWidget: type
          };
          config.type = RowCtor;
          config.properties = Properties.extend(that.properties(), {
            allowHyperlink: false, ordBase: ordBase
          }, config.properties);

          return config;
        });
    };

    return builder;
  };

  return MgrSheet;
});
