/*
 * @copyright 2016 Tridium, Inc. All Rights Reserved.
 */

/**
 * @private
 * @module nmodule/webEditors/rc/wb/mgr/commands/LearnModeCommand
 */
define(['Promise',
        'bajaux/commands/ToggleCommand',
        'nmodule/webEditors/rc/wb/mixin/mixinUtils',
        'nmodule/webEditors/rc/wb/mgr/commands/MgrCommand'], function (
        Promise,
        ToggleCommand,
        mixinUtils,
        addMgrCommandSupport) {

  'use strict';

  /**
   * Toggle command for showing or hiding the learn mode on the manager view.
   * Starting a discovery will automatically set this command to the 'on'
   * state. This command will not be shown in the manager's action bar, only
   * on the tool bar.
   *
   * For this command to function correctly, the manager must have the `MgrLearn`
   * functionality mixed in.
   *
   * @private
   * @class
   * @extends module:bajaux/commands/ToggleCommand
   * @alias module:nmodule/webEditors/rc/wb/mgr/commands/LearnModeCommand
   * @mixes module:nmodule/webEditors/rc/wb/mgr/commands/MgrCommand
   * @param {module:nmodule/webEditors/rc/wb/mgr/MgrLearn} mgr
   */
  var LearnModeCommand = function LearnModeCommand(mgr) {
    var that = this;

    ToggleCommand.call(that, {
      module: 'webEditors',
      lex: 'commands.mgr.learnMode',
      selected: false,
      enabled: true,

      /**
       * Function called when the Command is toggled. This will hide or show
       * the discovery pane in the manager view.
       *
       * @alias module:nmodule/webEditors/rc/wb/mgr/commands/LearnModeCommand#invoke
       * @returns {Promise}
       */
      func: function () {
        this.toggle();

        if (mixinUtils.hasMixin(mgr, 'MGR_LEARN')) {
          mgr.$toggleLearnPane(that.isSelected());
          return Promise.resolve();
        }
        else {
          return Promise.reject(new Error('manager does not have learn support'));
        }
      }
    });

    addMgrCommandSupport(that);
    that.setShowInActionBar(false);
  };

  LearnModeCommand.prototype = Object.create(ToggleCommand.prototype);
  LearnModeCommand.prototype.constructor = LearnModeCommand;

  return LearnModeCommand;
});
