/**
 * @copyright 2016 Tridium, Inc. All Rights Reserved.
 */

/**
 * @private
 * @module nmodule/webEditors/rc/wb/mgr/commands/NewFolderCommand
 */
define(['lex!webEditors',
        'bajaux/commands/Command',
        'nmodule/webEditors/rc/fe/feDialogs',
        'nmodule/webEditors/rc/wb/mixin/mixinUtils'], function (
        lexs,
        Command,
        feDialogs,
        mixinUtils) {

  'use strict';

////////////////////////////////////////////////////////////////
// NewFolderCommand
////////////////////////////////////////////////////////////////

  var webEditorsLex = lexs[0];

  /**
   * Command for adding new folders to a driver manager view.
   * The type of folder to be created is defined by the manager
   * model, specified in the constructor. This command requires
   * the manager's model to have the `folderMgrModelMixin` applied.
   *
   * @private
   * @class
   * @extends module:bajaux/commands/Command
   * @alias module:nmodule/webEditors/rc/wb/mgr/commands/NewFolderCommand
   * @param {module:nmodule/webEditors/rc/wb/mgr/Manager} manager
   */
  var NewFolderCommand = function NewFolderCommand(manager) {

    Command.call(this, {
      module: 'webEditors',
      lex: 'commands.mgr.newFolder',

      /**
       * Prompt the user with a dialog for a new folder name, then add a new folder
       * to the root subject component.
       *
       * @alias module:nmodule/webEditors/rc/wb/mgr/commands/NewFolderCommand#invoke
       * @returns {Promise}
       */
      func: function () {
        var mgrModel = manager.getModel();

        if (!mixinUtils.hasMixin(mgrModel, 'FOLDER_MGR_MODEL')) {
          throw new Error('Manager model type must have the folderMgrModelMixin applied.');
        }

        return feDialogs.showFor({
          value: webEditorsLex.get('newFolder.dlg.defVal'),
          title: webEditorsLex.get('newFolder.dlg.title')
        })
        .then(function (name) {
          if (name !== null) {
            return mgrModel.createFolder(name);
          }
        });
      }
    });
  };

  NewFolderCommand.prototype = Object.create(Command.prototype);
  NewFolderCommand.prototype.constructor = NewFolderCommand;

  return NewFolderCommand;
});
