/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author E542627
 */

/*jshint browser: true */

define(['jquery',
        'Promise',
        'underscore',
        'nmodule/webEditors/rc/wb/mixin/mixinUtils'], function (
         $,
         Promise,
         _,
         mixinUtils) {

  'use strict';

  var isSmoothScrollSupported = 'scrollBehavior' in document.documentElement.style,
    applyMixin = mixinUtils.applyMixin,
    MIXIN_NAME = 'stalkerSupport';

  function scroll(elem, options) {
    if (isSmoothScrollSupported) {
      elem.scrollBy(options);
    } else {
      elem.scrollTop += options.top;
    }
  }

  /**
   * When applied to a `PropertySheet`, adds a stalker showing the parent slot
   * of the row you are currently scrolled to.
   *
   * @private
   * @exports nmodule/webEditors/rc/wb/mixin/PropertySheetStalkerSupport
   * @mixin
   */
  var exports = {
    $getTableStalker: function () {
      return this.jq().children('.PropertySheet-table-stalker-container')
        .children('.PropertySheet-table-stalker');
    },

    $getCurrentParentRow: function () {
      var topRow = _.find(this.$getTableContainer().find('.PropertySheetRow'), function (el) {
        el = $(el);
        return el.is(':visible') && el.position().top >= 0;
      });
      if (topRow) {
        var expansion = $(topRow).closest('.PropertySheetRow-inline-expansion');
        if (expansion.is(':visible')) {
          return expansion.prev();
        }
      }
      return $();
    },

    $updateStalker: function () {
      var parentRow = this.$getCurrentParentRow(),
        stalker = this.$getTableStalker().empty();

      while (parentRow.length) {
        var crumb = $('<span/>')
          .text(parentRow.attr('title'))
          .attr('data-y', parentRow.position().top);
        stalker.append(crumb);
        parentRow = parentRow.closest('.PropertySheetRow-inline-expansion').prev();
      }

      stalker.toggle(!!stalker.children().length);
    }
  };

  var addStalkerSupport = function (target) {
    if (!applyMixin(target, MIXIN_NAME, exports)) {
      return;
    }

    var doDestroy = target.doDestroy,
      doInitialize = target.doInitialize,
      doLayout = target.doLayout;

    target.doInitialize = function (dom) {
      var that = this,
        containerScrollHandler = that.$containerScrollHandler = _.throttle(function () {
          that.$updateStalker();
        }, 150, { leading: false });

      dom.on('click', '.PropertySheet-table-stalker span', function (e) {
        scroll(that.$getTableContainer()[0], {
          top: $(e.target).data('y') - that.$getTableStalker().outerHeight(),
          behavior: 'smooth'
        });
      });

      return Promise.resolve(doInitialize.apply(that, arguments))
        .then(function () {
          that.$getTableContainer().on('scroll', containerScrollHandler);
        });
    };

    target.doLayout = function () {
      var that = this;
      return doLayout.apply(that, arguments)
        .then(function () {
          that.$updateStalker();
        });
    };

    target.doDestroy = function () {
      this.$getTableContainer().off('scroll', this.$containerScrollHandler);
      return doDestroy.apply(this, arguments);
    };
  };

  return addStalkerSupport;
});
