/*
 * @copyright 2016 Tridium, Inc. All Rights Reserved.
 * @author Vikram Nagulan
 */

/**
 * @private
 * @module nmodule/webEditors/rc/wb/table/pagination/PaginationHandler
 */
define([], function () {
  'use strict';

  /**
   * Used in conjunction with a Table widget and provides a handler to
   * handle page events.
   *
   * @private
   * @class
   * @see module:nmodule/webEditors/rc/wb/table/Table
   * @example
   *   new Table({paginationHandler: myHandler})
   */
  var PaginationHandler = function PaginationHandler() {
  };

  //TODO: pageChanged is never actually called by the pagination API - does this belong here?
  /**
   * Fetches new page results for the passed in page index
   * @see module:nmodule/webEditors/rc/wb/table/pagination/PaginationWidget
   * @abstract
   * @param {Table} table The source table
   * @param {Number} pageIndex The page index to change to
   * @return {module:nmodule/webEditors/rc/wb/table/model/TableModel|Promise} new
   * table model, or promise that resolves to one
   */
  PaginationHandler.prototype.pageChanged = function (table, pageIndex) {
    throw new Error("pageChanged not implemented!");
  };

  /**
   * Returns a table model. Subclasses will implement this method and return a TableModel
   * constructed from the value loaded in to a bajaux widget (e.g BqlGridTable)
   * @return {Promise.<module:nmodule/webEditors/rc/wb/table/model/TableModel>} promise
   * that resolves to a TableModel or rejects with an error
   */
  PaginationHandler.prototype.getTableModel = function () {
    throw new Error("getTableModel not implemented!");
  };

  /**
   * Perform any finalize activities here
   * @abstract
   */
  PaginationHandler.prototype.destroy = function () {
    throw new Error("destroy not implemented!");
  };

  return PaginationHandler;
});
