/*
 * @copyright 2016 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * @private
 * @module nmodule/webEditors/rc/wb/tree/JSONTreeNode
 */
define(['underscore',
        'nmodule/webEditors/rc/wb/tree/TreeNode'], function (
         _,
         TreeNode) {

  'use strict';

  function isObjectOrArray(v) { return v && typeof v === 'object'; }

  /**
   * @private
   * @class
   * @alias module:nmodule/webEditors/rc/wb/tree/JSONTreeNode
   * @extends module:nmodule/webEditors/rc/wb/tree/TreeNode
   * @param {String|Object} name the name of the tree node. Or, you can just
   * pass a single argument, an object or array, and the tree node hierarchy
   * will be built out from that value.
   * @param {*} obj the JSON value represented by this node
   */
  var JSONTreeNode = function JSONTreeNode(name, obj) {
    if (isObjectOrArray(name)) {
      obj = name;
      name = 'root';
    } else if (arguments.length < 2 ||
      !(typeof name === 'string' || typeof name === 'number')) {
      throw new Error('name required');
    }

    var kids;
    if (isObjectOrArray(obj)) {
      kids = _.map(obj, function (value, key) {
        return new JSONTreeNode(key, value);
      });
    } else {
      kids = [];
    }

    name = String(name);
    this.$value = obj;
    TreeNode.call(this, name, name, kids);
  };
  JSONTreeNode.prototype = Object.create(TreeNode.prototype);
  JSONTreeNode.prototype.constructor = JSONTreeNode;

  /**
   * The JSON value represented by this node.
   * @returns {*}
   */
  JSONTreeNode.prototype.value = function () { return this.$value; };

  /**
   * If this value does not represent an object or array, it cannot have child
   * nodes.
   * @returns {boolean} true iff the value is an object or array
   */
  JSONTreeNode.prototype.mayHaveKids = function () {
    return !!isObjectOrArray(this.value());
  };

  return JSONTreeNode;
});
